<?php
//  ---------------------------------------------------------------------------------
//    _____   _             _                   _   _____
//   |  __ \ | |           | |    /\           (_) |  __ \
//   | |__) || |__    __ _ | |   /  \    _ __   _  | |__) |_ __  ___
//   |  ___/ | '_ \  / _` || |  / /\ \  | '_ \ | | |  ___/| '__|/ _ \
//   | |     | | | || (_| || | / ____ \ | |_) || | | |    | |  | (_) |
//   |_|     |_| |_| \__,_||_|/_/    \_\| .__/ |_| |_|    |_|   \___/
//                                      | |
//                                      |_|
//   PhalApi Pro 专业版
//   广州果创网络科技有限公司
//
//  ---------------------------------------------------------------------------------
//
//   一、协议的许可和权利
//    1. 您可以在完全遵守本协议的基础上，将本软件应用于商业用途；
//    2. 您可以在协议规定的约束和限制范围内修改本产品源代码或界面风格以适应您的要求；
//    3. 您拥有使用本产品中的全部内容资料、商品信息及其他信息的所有权，并独立承担与其内容相关的
//       法律义务；
//    4. 获得商业授权之后，您可以将本软件应用于商业用途，自授权时刻起，在技术支持期限内拥有通过
//       指定的方式获得指定范围内的技术支持服务；
//
//   二、协议的约束和限制
//    1. 未获商业授权之前，禁止将本软件用于商业用途（包括但不限于企业法人经营的产品、经营性产品
//       以及以盈利为目的或实现盈利产品）；
//    2. 未获商业授权之前，禁止在本产品的整体或在任何部分基础上发展任何派生版本、修改版本或第三
//       方版本用于重新开发；
//    3. 如果您未能遵守本协议的条款，您的授权将被终止，所被许可的权利将被收回并承担相应法律责任；
//
//   三、有限担保和免责声明
//    1. 本软件及所附带的文件是作为不提供任何明确的或隐含的赔偿或担保的形式提供的；
//    2. 用户出于自愿而使用本软件，您必须了解使用本软件的风险，在尚未获得商业授权之前，我们不承
//       诺提供任何形式的技术支持、使用担保，也不承担任何因使用本软件而产生问题的相关责任；
//    3. 广州果创网络科技有限公司不对使用本产品构建的商城中的内容信息承担责任，但在不侵犯用户隐
//       私信息的前提下，保留以任何方式获取用户信息及商品信息的权利；
//
//   有关本产品最终用户授权协议、商业授权与技术服务的详细内容，均由广州果创网络科技有限公司独家
//   提供。广州果创网络科技有限公司拥有在不事先通知的情况下，修改授权协议的权力，修改后的协议对
//   改变之日起的新授权用户生效。电子文本形式的授权协议如同双方书面签署的协议一样，具有完全的和
//   等同的法律效力。您一旦开始修改、安装或使用本产品，即被视为完全理解并接受本协议的各项条款，
//   在享有上述条款授予的权力的同时，受到相关的约束和限制。协议许可范围以外的行为，将直接违反本
//   授权协议并构成侵权，我们有权随时终止授权，责令停止损害，并保留追究相关责任的权力。
//
//  ---------------------------------------------------------------------------------

namespace Mall\Domain;

use Base\Domain\Base;
use Mall\Model\Mall\Order as OrderModel;
use Mall\Model\Mall\Product as ProductModel;
use Base\Domain\User;
use PhalApi\Exception\BadRequestException;

class Order extends Base{

    /** ----------------------------- 开放平台 ----------------------------- **/

    //订单统计
    public function myOrderCount(){
        $model = new OrderModel();
        return $model->count(['member_id'=>$this->uid]);
    }

    //待支付订单
    public function myOrderCountWaile(){
        $model = new OrderModel();
        return $model->count(['member_id'=>$this->uid,'order_status'=>OrderModel::ORDER_STATUS_NEW]);
    }


    public function getOrderInfo($order_id) {
        $model = new OrderModel();
        $order = $model->getDataBy('order_id', $order_id);
        return $order ? $order : null;
    }

    public function checkOrderPay($order_id) {
        $uid = \PhalApi\DI()->context->getUid();
        $model = new OrderModel();
        $order = $model->getData('order_id = :order_id AND member_id = :member_id', array(':order_id' => $order_id, ':member_id' => $uid));
        if (empty($order)) {
            throw new BadRequestException(\PhalApi\T('order not exists'));
        }

        if ($order && in_array($order['order_status'], array(OrderModel::ORDER_STATUS_PAID, OrderModel::ORDER_STATUS_FINISH))) {
            return true;
        }

        // 同步查询
        $payDomain = new Pay();
        if ($payDomain->checkPay($order)) {
            $payNotify = new PayNotify($order);
            $payNotify->handle($order['pay_type'], 'pay', 'sync', array());
            return true;
        }

        return false;
    }

    // 前台，创建订单
    public function createOrder($product_id, $num, $note) {
        $productDomain = new Product();
        $product = $productDomain->getAvailableProduct($product_id);
        if (!$product) {
            return null;
        }

        $userDomain = new User();
        $uid = \PhalApi\DI()->context->getUid();

        $configDomain = new Config();

        $newOrder = array(
            'member_id' => $uid,
            'order_id' => date('YmdHis') . rand(10000 ,99999),
            'order_name' => $product['product_name'],
            'price' => floatval($product['price']) * $num,
            'add_time' => date('Y-m-d H:i:s'),
            'product_key' => $product['product_key'],
            'product_type' => $product['product_type'],
            'product_expire_time' => $product['expire_time'],
            'product_amount' => $product['amount'] * $num,
            'order_status' => OrderModel::ORDER_STATUS_NEW,
            'product_id' => $product_id,
            'product_expire_time_desc' => $product['expire_time_desc'],
            'num' => $num,
            'note' => $note,
            'username' => $userDomain->getUsername($uid),
            'pay_type' => $configDomain->getDefaultPayType(),
        );

        // 价格为0，直接支付成功
        if ($newOrder['price'] == 0) {
            $newOrder['order_status'] = OrderModel::ORDER_STATUS_PAID;
        }

        $model = new OrderModel(); 
        $id = $model->insert($newOrder);
        return $id ? $newOrder['order_id'] : null;
    }

    public function updatePayType($id, $pay_type) {
        $model = new OrderModel();
        return $model->update($id, array('pay_type' => $pay_type));
    }

    public function getMyOrderList($page, $perpage, $uid = null) {
        $uid = $uid ? $uid : \PhalApi\DI()->context->getUid();

        $model = new OrderModel();
        $select = '*';
        $data = $model->getListEx('member_id = :member_id', array(':member_id' => $uid), $select, 'id DESC', $page, $perpage);

        $configDomain = new Config();
        $payConfig = $configDomain->getPayConfig();

        foreach ($data['items'] as &$itRef) {
            $itRef['order_status_name'] = \PhalApi\T('order_status_' . $itRef['order_status']);
            $itRef['pay_type_desc'] = isset($payConfig[$itRef['pay_type']]) ? $payConfig[$itRef['pay_type']]['name'] : $itRef['pay_type'];
        }
        return $data;
    }

    /** ----------------------------- 管理后台 ----------------------------- **/

    public function updateOrder($id, $order, $op) {
        $model = new OrderModel();
        $oldOrder = $model->get($id);
        if (!$model) {
            return false;
        }

        $flow = new Flow();
        // 是否需要发放套餐
        if ($op == 'send' && empty($oldOrder['is_deliver'])) {
            $flow->sendOrderFlow($oldOrder);

            // 标识为已发放
            $order['is_deliver'] = 1;
            // 后台发放时，支付时间更新，修复套餐时间不生效问题
            $order['pay_time'] = date('Y-m-d H:i:s');
        }

        // 是否需要回收套餐
        if ($op == 'back') {
            $flow->takebackOrderFlow($oldOrder['order_id']);
        }

        $isUpdate = $model->update($id, $order);
        return true;
    }

    public function getOrder($id) {
        $model = new OrderModel();
        $order = $model->get($id);
        return $order ? $order : null;
    }

    public function deleteOrder($id) {
        $model = new OrderModel();
        return $model->delete($id) ? true : false;
    }

    public function getOrderList($page, $perpage){
        $model = new OrderModel();
        $select = '*';
        $data = $model->getListEx(NULL, array(), $select, 'id DESC', $page, $perpage);

        $configDomain = new Config();
        $payConfig = $configDomain->getPayConfig();

        foreach ($data['items'] as &$itRef) {
            $itRef['order_status_name'] = \PhalApi\T('order_status_' . $itRef['order_status']);
            $itRef['pay_type_desc'] = isset($payConfig[$itRef['pay_type']]) ? $payConfig[$itRef['pay_type']]['name'] : $itRef['pay_type'];
        }
        return $data;
    }

    public function getOrderStatusList($isMap = true) {
        $map = OrderModel::getOrderStatusList();

        if (!$isMap) {
            $list = array();
            foreach ($map as $key => $val) {
                $list[] = array('name' => $val, 'key' => $key);
            }
            $map = $list;
        }

        return $map;
    }
}
